/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.callstack;

import tools.descartes.prisma.agent.wildfly.parameters.ValueCharacterization;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;

public class OperationRecord {

	private final ComponentRecord component;
	private final String operation;
	private final String interf;
	private final String protocol;
	private final InvocationRecord caller;
	private String stringRepresentation;

	private ValueCharacterization value = null;
	private String[] observedParameters = new String[0];
	private int[] observedParametersIndeces = new int[0];
	private InvocationRecord[] groups = new InvocationRecord[1];
		
	public OperationRecord(MonitoringService monitoringService, InvocationRecord caller, ComponentRecord component,
			String interf,
			String protocol, String operation, ValueCharacterization value) {
		this.caller = caller;
		this.component = component;
		this.interf = interf;
		this.protocol = protocol;
		this.operation = operation;
		this.value = value;
		if (value != null) {
			observedParameters = new String[1];
			observedParametersIndeces = new int[1];
			observedParameters[0] = value.getParameterName();
			observedParametersIndeces[0] = value.getParameterIndex();
			groups = new InvocationRecord[value.getNumberOfValues()];
			for (int i = 0; i < groups.length; i++) {
				groups[i] = new InvocationRecord(monitoringService, this, 0);
			}
		} else {
			// Default group
			groups[0] = new InvocationRecord(monitoringService, this, 0);
		}
	}
	
	public static String getEndpointIdentification(ComponentRecord component, String interf, String protocol,
			String operation) {
		StringBuilder name = new StringBuilder();
		name.append(protocol).append("://");
		name.append(component.toString());
		name.append("/");
		name.append(interf);
		name.append("/");
		name.append(operation);
		return name.toString();
	}

	public InvocationRecord getCaller() {
		return caller;
	}

	public ComponentRecord getComponent() {
		return component;
	}

	public String getOperationName() {
		return operation;
	}
	
	public String getProtocol() {
		return protocol;
	}

	public String getInterfaceName() {
		return interf;
	}

	public boolean areParametersMonitored() {
		return observedParameters.length > 0;
	}

	public int[] getObservedParameterIndeces() {
		return observedParametersIndeces;
	}

	public String[] getObservedParameterNames() {
		return observedParameters;
	}
	
	public InvocationRecord startInvocation() {
		return groups[0];
	}

	public InvocationRecord startInvocation(Object... values) {
		int idx = value.getValueIndex(values[0]);
		return groups[idx];
	}
	
	public InvocationRecord[] getInvocations() {
		return groups;
	}

	@Override
	public String toString() {
		if (stringRepresentation == null) {
			stringRepresentation = getEndpointIdentification(component, interf, protocol, operation);
		}
		return stringRepresentation;
	}

}
