/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.filters;

import java.util.Arrays;
import java.util.Collection;

import org.jboss.as.controller.AttributeDefinition;
import org.jboss.as.controller.PathElement;
import org.jboss.as.controller.PersistentResourceDefinition;
import org.jboss.as.controller.ReloadRequiredRemoveStepHandler;
import org.jboss.as.controller.SimpleAttributeDefinitionBuilder;
import org.jboss.as.controller.registry.AttributeAccess;
import org.jboss.dmr.ModelNode;
import org.jboss.dmr.ModelType;

import tools.descartes.prisma.agent.wildfly.extension.SubsystemExtension;

public class FilterDefinition extends PersistentResourceDefinition {

	public static final AttributeDefinition COMPONENT_PATTERN =
            new SimpleAttributeDefinitionBuilder(SubsystemExtension.COMPONENT_PATTERN, ModelType.STRING)
                    .setAllowExpression(true)
                    .setXmlName(SubsystemExtension.COMPONENT_PATTERN)
                    .setFlags(AttributeAccess.Flag.RESTART_ALL_SERVICES)
                    .setDefaultValue(new ModelNode(".*"))
                    .setAllowNull(false)
                    .build();

	public static final AttributeDefinition INTERFACE_PATTERN =
            new SimpleAttributeDefinitionBuilder(SubsystemExtension.INTERFACE_PATTERN, ModelType.STRING)
                    .setAllowExpression(true)
                    .setXmlName(SubsystemExtension.INTERFACE_PATTERN)
                    .setFlags(AttributeAccess.Flag.RESTART_ALL_SERVICES)
                    .setDefaultValue(new ModelNode(".*"))
                    .setAllowNull(false)
                    .build();

	public static final AttributeDefinition OPERATION_PATTERN =
            new SimpleAttributeDefinitionBuilder(SubsystemExtension.OPERATION_PATTERN, ModelType.STRING)
                    .setAllowExpression(true)
                    .setXmlName(SubsystemExtension.OPERATION_PATTERN)
                    .setFlags(AttributeAccess.Flag.RESTART_ALL_SERVICES)
                    .setDefaultValue(new ModelNode(".*"))
                    .setAllowNull(false)
                    .build();
	
	public static final FilterDefinition INSTANCE = new FilterDefinition();
	
	private FilterDefinition() {
		super(PathElement.pathElement(SubsystemExtension.FILTER), SubsystemExtension.getResourceDescriptionResolver(SubsystemExtension.FILTER), FilterAdd.INSTANCE, ReloadRequiredRemoveStepHandler.INSTANCE);
	}

	@Override
	public Collection<AttributeDefinition> getAttributes() {
		return Arrays.asList(COMPONENT_PATTERN, INTERFACE_PATTERN, OPERATION_PATTERN);
	}
	
}
