/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.interceptors;

import javax.jms.Destination;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.Topic;

import org.jboss.as.ee.component.ComponentDescription;
import org.jboss.invocation.InterceptorContext;

import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;

public class EJBMessageInterceptor extends AbstractInterceptor {

	private MonitoringService monitoringService;

	public EJBMessageInterceptor(MonitoringService monitoringService, ComponentDescription description) {
		super(monitoringService, description.getComponentName(), description, "javax.jms.MessageListener", "jms",
				"onMessage");
		this.monitoringService = monitoringService;
	}

	@Override
	public Object processInvocation(InterceptorContext ctx) throws Exception {
		long startTime = System.nanoTime();

		InvocationRecord invocation = startInvocation(ctx);

		// Determine where this message came from and a queue component
		Message msg = (Message) ctx.getParameters()[0];
		Destination dest = msg.getJMSDestination();
		if (dest != null) {
			String destName = null;
			String destClass = null;
			if (dest instanceof Queue) {
				destName = ((Queue) dest).getQueueName();
				destClass = "javax.jms.Queue";
			} else if (dest instanceof Topic) {
				destName = ((Topic) dest).getTopicName();
				destClass = "javax.jms.Topic";
			}
			ComponentRecord destinationCrec = monitoringService.requireComponent("", "", "",
					destName);
			if (destinationCrec.getComponentClass() == null) {
				destinationCrec.setComponentClass(destClass);
			}
			InvocationRecord destIrec = monitoringService
					.requireOperation(destinationCrec, "javax.jms.MessageProducer", "jms", "send")
					.startInvocation();
			OperationRecord destCalledOrec = destIrec
					.requireEndpoint(component, interfaceName, "jms", operation.getOperationName());
			// This is an asynchronous invocation so the execution time from the
			// perspective of the caller is zero.
			destCalledOrec.getInvocations()[invocation.getValueBucket()].finishInvocation(0, 0, 0);
		}
		try {
			return ctx.proceed();
		} finally {
			finishInvocation(startTime);
		}
	}

}
