/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.interceptors.jms;

import javax.jms.Connection;
import javax.jms.ConnectionConsumer;
import javax.jms.ConnectionMetaData;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.ServerSessionPool;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;

public class ConnectionDelegate implements QueueConnection, TopicConnection {

	private final Connection delegate;

	public ConnectionDelegate(Connection delegate) {
		super();
		this.delegate = delegate;
	}

	public Session createSession(boolean transacted, int acknowledgeMode) throws JMSException {
		return new SessionDelegate(delegate.createSession(transacted, acknowledgeMode));
	}

	public Session createSession(int sessionMode) throws JMSException {
		return new SessionDelegate(delegate.createSession(sessionMode));
	}

	public Session createSession() throws JMSException {
		return new SessionDelegate(delegate.createSession());
	}

	public String getClientID() throws JMSException {
		return delegate.getClientID();
	}

	public void setClientID(String clientID) throws JMSException {
		delegate.setClientID(clientID);
	}

	public ConnectionMetaData getMetaData() throws JMSException {
		return delegate.getMetaData();
	}

	public ExceptionListener getExceptionListener() throws JMSException {
		return delegate.getExceptionListener();
	}

	public void setExceptionListener(ExceptionListener listener) throws JMSException {
		delegate.setExceptionListener(listener);
	}

	public void start() throws JMSException {
		delegate.start();
	}

	public void stop() throws JMSException {
		delegate.stop();
	}

	public void close() throws JMSException {
		delegate.close();
	}

	public ConnectionConsumer createConnectionConsumer(Destination destination, String messageSelector,
			ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createConnectionConsumer(destination, messageSelector, sessionPool, maxMessages);
	}

	public ConnectionConsumer createSharedConnectionConsumer(Topic topic, String subscriptionName,
			String messageSelector, ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createSharedConnectionConsumer(topic, subscriptionName, messageSelector, sessionPool,
				maxMessages);
	}

	public ConnectionConsumer createDurableConnectionConsumer(Topic topic, String subscriptionName,
			String messageSelector, ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createDurableConnectionConsumer(topic, subscriptionName, messageSelector, sessionPool,
				maxMessages);
	}

	public ConnectionConsumer createSharedDurableConnectionConsumer(Topic topic, String subscriptionName,
			String messageSelector, ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createSharedDurableConnectionConsumer(topic, subscriptionName, messageSelector, sessionPool,
				maxMessages);
	}

	@Override
	public TopicSession createTopicSession(boolean transacted, int acknowledgeMode) throws JMSException {
		return new SessionDelegate(delegate.createSession(transacted, acknowledgeMode));
	}

	@Override
	public ConnectionConsumer createConnectionConsumer(Topic topic, String messageSelector,
			ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createConnectionConsumer(topic, messageSelector, sessionPool, maxMessages);
	}

	@Override
	public QueueSession createQueueSession(boolean transacted, int acknowledgeMode) throws JMSException {
		return new SessionDelegate(delegate.createSession(transacted, acknowledgeMode));
	}

	@Override
	public ConnectionConsumer createConnectionConsumer(Queue queue, String messageSelector,
			ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createConnectionConsumer(queue, messageSelector, sessionPool, maxMessages);
	}
}
