/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.interceptors.jms;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSContext;
import javax.jms.JMSException;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class ConnectionFactoryDelegate implements QueueConnectionFactory, TopicConnectionFactory {
	
	private final String jndiDelegate;
	private ConnectionFactory delegate = null;
	
	public ConnectionFactoryDelegate(String jndiDelegate) {
		this.jndiDelegate = jndiDelegate;
	}

	public Connection createConnection() throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection());
	}

	public Connection createConnection(String userName, String password) throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection(userName, password));
	}

	public JMSContext createContext() {
		return new JMSContextDelegate(getDelegate().createContext());
	}

	public JMSContext createContext(String userName, String password) {
		return new JMSContextDelegate(getDelegate().createContext(userName, password));
	}

	public JMSContext createContext(String userName, String password, int sessionMode) {
		return new JMSContextDelegate(getDelegate().createContext(userName, password, sessionMode));
	}

	public JMSContext createContext(int sessionMode) {
		return new JMSContextDelegate(getDelegate().createContext(sessionMode));
	}
	
	@Override
	public TopicConnection createTopicConnection() throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection());
	}

	@Override
	public TopicConnection createTopicConnection(String userName, String password) throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection(userName, password));
	}

	@Override
	public QueueConnection createQueueConnection() throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection());
	}

	@Override
	public QueueConnection createQueueConnection(String userName, String password) throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection(userName, password));
	}
	
	private ConnectionFactory getDelegate() {
		if (delegate == null) {
			Context ctx;
			try {
				ctx = new InitialContext();
				delegate = (ConnectionFactory) ctx.lookup(jndiDelegate);
			} catch (NamingException e) {
				throw new RuntimeException(e);
			}			
		}
		return delegate;
	}

}
