/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.parameters;

import java.util.Collection;

public class NullValueCharacterization implements ValueCharacterization {
	
	private final String name;
	private final int idx;
	private final String interfaceName;
	private final String operationName;
	
	public NullValueCharacterization(String interfaceName, String operationName, String name) {
		this(interfaceName, operationName, name, -1);
	}
	
	public NullValueCharacterization(String interfaceName, String operationName, String name, int idx) {
		this.name = name;
		this.idx = idx;
		this.interfaceName = interfaceName;
		this.operationName = operationName;
	}
	
	@Override
	public String getInterfaceName() {
		return interfaceName;
	}
	
	@Override
	public String getOperationName() {
		return operationName;
	}

	@Override
	public String getParameterName() {
		return name;
	}

	@Override
	public int getNumberOfValues() {
		return 2;
	}

	@Override
	public int getValueIndex(Object value) {
		if (value instanceof Collection) {
			return (value != null && !((Collection<?>)value).isEmpty()) ? 1 : 0;
		}
		return (value == null) ? 0 : 1;
	}
	
	@Override
	public String getValueIdentifier(int idx) {
		return (idx == 0) ? "Null" : "NotNull";
	}

	@Override
	public int getParameterIndex() {
		return idx;
	}

}
