/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.service;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.UUID;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.jboss.logging.Logger;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.ServiceName;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;
import org.jboss.msc.value.InjectedValue;

import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ComponentInstanceReference;
import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ModelVariableCharacterizationType;
import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ParameterdependenciesFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.BasicComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.ComposedStructure;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.CompositeComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Interface;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceRequiringRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.ProvidingDelegationConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RequiringDelegationConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Signature;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.AbstractAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCall;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCallAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.FineGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ForkAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.InternalAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopIterationCount;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ResourceDemand;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.system.SystemFactory;
import edu.kit.ipd.descartes.mm.deployment.DeploymentContext;
import edu.kit.ipd.descartes.mm.deployment.DeploymentFactory;
import edu.kit.ipd.descartes.mm.resourcelandscape.Container;
import edu.kit.ipd.descartes.mm.resourcelandscape.RuntimeEnvironment;
import edu.kit.ipd.descartes.mm.runtimeenvironmentclasses.RuntimeEnvironmentClasses;
import tools.descartes.librede.metrics.Aggregation;
import tools.descartes.librede.metrics.Metric;
import tools.descartes.librede.metrics.StandardMetrics;
import tools.descartes.librede.units.Dimension;
import tools.descartes.librede.units.RequestCount;
import tools.descartes.librede.units.Time;
import tools.descartes.librede.units.Unit;
import tools.descartes.prisma.agent.wildfly.Constants;
import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.filters.OperationPredicate;
import tools.descartes.prisma.core.MessageBus;
import tools.descartes.prisma.core.ModelRepository;
import tools.descartes.prisma.core.agent.AgentController;
import tools.descartes.prisma.core.agent.RemoveFilter;
import tools.descartes.prisma.core.impl.CDOModelRepository;
import tools.descartes.prisma.core.impl.RabbitMQMessageBus;
import tools.descartes.prisma.core.messages.ModelNotification;
import tools.descartes.prisma.core.scopes.AgentScope.NotificationHandler;
import tools.descartes.prisma.core.scopes.ApplicationScope;
import tools.descartes.prisma.core.util.ModelSkeletonUtil;
import tools.descartes.prisma.model.sensor.ComponentSensor;
import tools.descartes.prisma.model.sensor.Sensor;
import tools.descartes.prisma.model.sensor.SensorFactory;
import tools.descartes.prisma.model.skeleton.ModelSkeleton;
import tools.descartes.prisma.model.skeleton.SkeletonFactory;

public class ModelExtractionService extends AgentController implements Service<ModelExtractionService> {

	private class CallPaths {
		private Map<String, List<String>> paths = new HashMap<>();

		public void addPath(String interf, String assemblyCtx) {
			List<String> a = paths.get(interf);
			if (a == null) {
				a = new LinkedList<>();
				paths.put(interf, a);
			}
			a.add(assemblyCtx);
		}

		public int getIndex(String interf, String assemblyCtx) {
			List<String> a = paths.get(interf);
			if (a != null) {
				int idx = 0;
				for (String cur : a) {
					if (cur.equals(assemblyCtx)) {
						return idx;
					}
					idx++;
				}
			}
			return -1;
		}
	}

	private static final Logger log = Logger.getLogger(ModelExtractionService.class);

	private InjectedValue<MonitoringService> monitoring = new InjectedValue<>();

	private List<InjectedValue<FilterService>> filters = new CopyOnWriteArrayList<>();

	public static final ServiceName SERVICE_NAME = ServiceName.JBOSS.append("modelextraction", "modelextraction");

	public static ModelRepository repository;

	public static final ModelExtractionService INSTANCE = new ModelExtractionService();

	private final List<OperationPredicate> operationFilters = new LinkedList<>();

	private ModelSkeleton skeleton;

	private RuntimeEnvironment wildFlyContainer;

	private Map<String, CallPaths> callpaths = new HashMap<>();

	private Queue<AssemblyContext> undeployedAssemblyContexts = new LinkedList<>();

	private Queue<Object> updates = new ConcurrentLinkedQueue<>();

	public ModelExtractionService() {
		super();
	}

	public void init(String repositoryHost, String repositoryUser, String repositoryPassword, String scopePath) {
		MessageBus messageBus = new RabbitMQMessageBus(repositoryHost, repositoryUser, repositoryPassword);
		repository = new CDOModelRepository(repositoryHost, repositoryUser, repositoryPassword, messageBus);
		super.init(repository, messageBus, scopePath,
				getLocalHost().getHostName().toString() + "_" + UUID.randomUUID().toString());
	}

	@Override
	public ModelExtractionService getValue() throws IllegalStateException, IllegalArgumentException {
		return this;
	}

	@Override
	public void start(StartContext arg0) throws StartException {
		for (InjectedValue<FilterService> filter : filters) {
			operationFilters.add(filter.getValue().getPredicate());
		}

		skeleton = SkeletonFactory.eINSTANCE.createModelSkeleton();
		skeleton.setRepository(RepositoryFactory.eINSTANCE.createRepository());
		skeleton.setSystem(SystemFactory.eINSTANCE.createSystem());
		skeleton.setDeployment(DeploymentFactory.eINSTANCE.createDeployment());
		skeleton.setSensors(SensorFactory.eINSTANCE.createSensorRepository());

		setRemoveFilter(new RemoveFilter() {
			@Override
			public boolean isRemovedOnShutdown(EObject object) {
				if (object instanceof DeploymentContext) {
					// Deployment contexts are always obsolete when wildfly is
					// shut down
					return true;
				}
				if (object instanceof Sensor<?>) {
					// Sensor are not available anymore after the agent shuts
					// down.
					return true;
				}
				if (object instanceof AssemblyContext || object instanceof AssemblyConnector
						|| object instanceof ProvidingDelegationConnector
						|| object instanceof RequiringDelegationConnector || object instanceof InterfaceProvidingRole
						|| object instanceof InterfaceRequiringRole) {
					// These elements are only removed if they are contained in
					// a System or Subsystem. CompositeComponents should stay as
					// they are.
					return !(object.eContainer() instanceof CompositeComponent)
							&& !(object.eContainer() instanceof BasicComponent);
				}
				return false;
			}
		});



		try {
			start().get();
			initWildflyContainer();
			schedule(new ModelUpdater(), 1, TimeUnit.MINUTES);
		} catch (InterruptedException | ExecutionException e) {
			log.error("Error starting agent.", e);
		}
	}

	private void initWildflyContainer() {
		ApplicationScope applicationScope = (ApplicationScope) getContainerScope();
		// Listen for new runtime environments
		addNotificationHandler(applicationScope.getRuntimeEnvironmentInputPort(), "#", new NotificationHandler() {
			@Override
			public void handleNotification(ModelNotification event) {
				if (event.getNotificationType().equals(ModelNotification.NEW_NOTIFICATION)) {
					EObject instance = event.getInstance();
					if (instance instanceof RuntimeEnvironment) {
						addNewRuntimeEnvironment((RuntimeEnvironment) instance);
					}
				}
			}
		});

		// Check for existing runtime environment representing the WildFly
		// container
		for (Container curContainer : applicationScope.getContainer()) {
			if (curContainer instanceof RuntimeEnvironment) {
				addNewRuntimeEnvironment((RuntimeEnvironment) curContainer);
			}
		}
	}

	public void addNewRuntimeEnvironment(RuntimeEnvironment env) {
		if (env.getOfClass() == RuntimeEnvironmentClasses.OPERATING_SYSTEM
				&& env.getName().equalsIgnoreCase(getLocalHost().getHostName())) {
			wildFlyContainer = ModelSkeletonUtil.adopt(ModelExtractionService.this, env);
		}
	}

	@Override
	public void stop(StopContext arg0) {
		try {
			stop().get();
		} catch (InterruptedException | ExecutionException e) {
			log.error("Error stopping agent.", e);
		}
	}

	public List<InjectedValue<FilterService>> getFilters() {
		return filters;
	}

	public void newComponentRecord(ComponentRecord record) {
		updates.add(record);
	}

	public void newOperationRecord(OperationRecord record) {
		updates.add(record);
	}

	public void newInvocationRecord(InvocationRecord record) {
		updates.add(record);
	}

	private class ModelUpdater implements Runnable {

		@Override
		public void run() {
			try {
				Object current;

				boolean changed = false;

				while ((current = updates.poll()) != null) {
					if (current instanceof InvocationRecord) {
						handleInvocation((InvocationRecord) current);
						changed = true;
					}
				}

				if (wildFlyContainer != null) {
					for (AssemblyContext curContext : undeployedAssemblyContexts) {
						DeploymentContext curDeployment = DeploymentFactory.eINSTANCE.createDeploymentContext();
						curDeployment.setName(toName(curContext.getName() + "@" + wildFlyContainer.getName()));
						curDeployment.setAssemblyContext(curContext);
						curDeployment.setResourceContainer(wildFlyContainer);
						ModelSkeletonUtil.insert(skeleton.getDeployment().getDeploymentContexts(), curDeployment);
						changed = true;
					}
					undeployedAssemblyContexts.clear();
				}

				if (changed) {
					apply(skeleton);
				}
			} catch (Throwable ex) {
				log.error("Error updating skeleton.", ex);
			}
		}

		private void handleInvocation(InvocationRecord record) {
			InvocationRecord caller = record.getOperation().getCaller();
			if (!isFiltered(record)) {
				if (caller != null) {
					if (!isFiltered(caller)) {
						// We have information on the incoming calls as well as
						// the outgoing calls.
						addComponent(caller, record);
					}
				} else {
					// no information about outgoing calls.
					addComponent(record, null);
				}
			}
		}
	}

	private boolean isFiltered(InvocationRecord record) {
		boolean filtered = false;
		String componentName = record.getOperation().getComponent().getComponentName();
		for (OperationPredicate filter : operationFilters) {
			if (filter.apply(componentName, record.getOperation().getInterfaceName(),
					record.getOperation().getOperationName())) {
				filtered = true;
				break;
			}
		}
		return filtered;
	}

	private void addComponent(InvocationRecord incomingIrec, InvocationRecord outgoingIrec) {
		ComponentRecord incomingCrec = incomingIrec.getOperation().getComponent();
		String incomingInterfaceName = incomingIrec.getOperation().getInterfaceName();
		boolean isRemoteIncomingCall = isRemoteCall(incomingIrec);
		ComponentRecord outgoingCrec = null;
		String outgoingInterfaceName = null;
		if (outgoingIrec != null) {
			outgoingCrec = outgoingIrec.getOperation().getComponent();
			outgoingInterfaceName = outgoingIrec.getOperation().getInterfaceName();
		}

		// 1. add the incoming interface to the repository
		Interface incomingInterface = RepositoryFactory.eINSTANCE.createInterface();
		incomingInterface.setName(toName(incomingInterfaceName));
		incomingInterface = ModelSkeletonUtil.insert(skeleton.getRepository().getInterfaces(), incomingInterface);

		Signature incomingSignature = RepositoryFactory.eINSTANCE.createSignature();
		incomingSignature.setName(toName(getSignatureName(incomingIrec)));
		incomingSignature = ModelSkeletonUtil.insert(incomingInterface.getSignatures(), incomingSignature);
		
		// 2. add the outgoing interface to the repository (if applicable)
		Interface outgoingInterface = null;
		Signature outgoingSignature = null;
		if (outgoingIrec != null) {
			outgoingInterface = RepositoryFactory.eINSTANCE.createInterface();
			outgoingInterface.setName(toName(outgoingInterfaceName));
			outgoingInterface = ModelSkeletonUtil.insert(skeleton.getRepository().getInterfaces(), outgoingInterface);
			outgoingSignature = RepositoryFactory.eINSTANCE.createSignature();
			outgoingSignature.setName(toName(getSignatureName(outgoingIrec)));
			outgoingSignature = ModelSkeletonUtil.insert(outgoingInterface.getSignatures(), outgoingSignature);
		}

		// 3. Create the implementation component for the caller component
		BasicComponent implementationComponent = RepositoryFactory.eINSTANCE.createBasicComponent();
		implementationComponent.setName(toName(incomingCrec.getComponentClass()));
		implementationComponent = (BasicComponent) ModelSkeletonUtil.insert(skeleton.getRepository().getComponents(),
				implementationComponent);
		String providingPortName = getInterfaceProvidingRoleName(incomingIrec, "");
		InterfaceProvidingRole callerProvidingRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
		callerProvidingRole.setName(toName(providingPortName));
		callerProvidingRole.setInterface(incomingInterface);
		callerProvidingRole = ModelSkeletonUtil.insert(implementationComponent.getInterfaceProvidingRoles(),
				callerProvidingRole);
		InterfaceRequiringRole callerRequiringRole = null;
		int callerRoleIdx = 0;
		if (outgoingInterface != null) {
			// Here we check whether there are external calls for the same
			// interface
			// but to different target assembly contexts. If yes, we need to
			// distinguish the two requiring ports
			callerRoleIdx = getRequiringPortIndex(incomingCrec, outgoingIrec.getOperation());
			String roleName = getInterfaceRequiringRoleName(outgoingIrec, "");
			if (callerRoleIdx > 0) {
				roleName = roleName + "/" + callerRoleIdx;
			}
			callerRequiringRole = RepositoryFactory.eINSTANCE.createInterfaceRequiringRole();
			callerRequiringRole.setName(toName(roleName));
			callerRequiringRole.setInterface(outgoingInterface);
			callerRequiringRole = ModelSkeletonUtil.insert(implementationComponent.getInterfaceRequiringRoles(),
					callerRequiringRole);
		}
		FineGrainedBehavior behavior = createComponentInternalBehavior(implementationComponent, callerProvidingRole,
				incomingSignature);
		ExternalCall externalCall = null;
		if (callerRequiringRole != null) {
			externalCall = addExternalCall(incomingCrec.getComponentClass(), implementationComponent, behavior,
					callerRequiringRole, outgoingSignature, outgoingIrec);
		}
		RepositoryComponent callerComponent = implementationComponent;

		// 4. Now add the called component (if applicable)
		InterfaceProvidingRole calledProvidingRole = null;
		RepositoryComponent calledComponent = null;
		boolean differentModule = false;
		if (callerRequiringRole != null) {
			differentModule = isDifferentModule(incomingCrec, outgoingCrec);
			calledComponent = createCalledComponent(outgoingCrec, differentModule);
			String calledProvingRoleName;
			if (calledComponent instanceof CompositeComponent) {
				calledProvingRoleName = getInterfaceProvidingRoleName(outgoingIrec,
						outgoingIrec.getOperation().getComponent().getComponentName());
			} else {
				calledProvingRoleName = getInterfaceProvidingRoleName(outgoingIrec, "");
			}

			calledProvidingRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
			calledProvidingRole.setName(toName(calledProvingRoleName));
			calledProvidingRole.setInterface(outgoingInterface);
			calledProvidingRole = ModelSkeletonUtil.insert(calledComponent.getInterfaceProvidingRoles(),
					calledProvidingRole);

			if (differentModule && outgoingIrec.getOperation().getProtocol().equals("java")) {
				// By default, local interface roles are not exposed on the
				// composite component. However, in certain cases local calls
				// between modules of the same application may exist. In this
				// case we explicitly expose this interface here and add the
				// required ProvidingDelegationConnector.
				addLocalCallToModule((CompositeComponent) calledComponent, calledProvidingRole, outgoingCrec,
						outgoingIrec);
			} else if (outgoingIrec.getOperation().getProtocol().equals("jdbc")) {
				addDataSource((CompositeComponent) calledComponent, calledProvidingRole, outgoingSignature);
			}
		}

		// 5. Now wrap the implementation in a composite component (if
		// necessary)
		CompositeComponent compositeComponent = createCompositeComponent(incomingCrec, callerComponent);
		AssemblyContext innerAssemblyContext = null;
		InterfaceProvidingRole innerCallerProvidingRole = callerProvidingRole;
		if (compositeComponent != null) {

			// Create an assembly context for embedding the component and
			// connect its interface providing role with a role on the composite
			// component
			innerAssemblyContext = addAssemblyContext(compositeComponent, incomingCrec, implementationComponent);

			if (isRemoteIncomingCall) {
				String providingRemotePortName = getInterfaceProvidingRoleName(incomingIrec,
						incomingCrec.getComponentName());
				InterfaceProvidingRole remoteProvidingRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
				remoteProvidingRole.setName(toName(providingRemotePortName));
				remoteProvidingRole.setInterface(incomingInterface);
				remoteProvidingRole = ModelSkeletonUtil.insert(compositeComponent.getInterfaceProvidingRoles(),
						remoteProvidingRole);

				ProvidingDelegationConnector providingConnector = RepositoryFactory.eINSTANCE
						.createProvidingDelegationConnector();
				providingConnector.setName(toName(innerAssemblyContext.getName()));
				providingConnector.setOuterInterfaceProvidingRole(remoteProvidingRole);
				providingConnector.setInnerInterfaceProvidingRole(callerProvidingRole);
				providingConnector.setAssemblyContext(innerAssemblyContext);
				ModelSkeletonUtil.insert(compositeComponent.getProvidingDelegationConnectors(), providingConnector);

				// IMPORTANT: from now on use this as the target of assembly
				// connectors.
				callerProvidingRole = remoteProvidingRole;
			}

			InterfaceRequiringRole remoteRequiringRole = null;
			if (callerRequiringRole != null) {
				if (differentModule) {
					// Only add a interface requiring role to the composite
					// component if it is a remote call, i.e., a call to another
					// composite component.
					String requiringRemotePortName = getInterfaceRequiringRoleName(outgoingIrec,
							incomingCrec.getComponentName());
					if (callerRoleIdx > 0) {
						requiringRemotePortName = requiringRemotePortName + "/" + callerRoleIdx;
					}
					remoteRequiringRole = RepositoryFactory.eINSTANCE.createInterfaceRequiringRole();
					remoteRequiringRole.setName(toName(requiringRemotePortName));
					remoteRequiringRole.setInterface(outgoingInterface);
					remoteRequiringRole = ModelSkeletonUtil.insert(compositeComponent.getInterfaceRequiringRoles(),
							remoteRequiringRole);

					RequiringDelegationConnector requiringConnector = RepositoryFactory.eINSTANCE
							.createRequiringDelegationConnector();
					requiringConnector.setName(toName(innerAssemblyContext.getName()));
					requiringConnector.setOuterInterfaceRequiringRole(remoteRequiringRole);
					requiringConnector.setInnerInterfaceRequiringRole(callerRequiringRole);
					requiringConnector.setAssemblyContext(innerAssemblyContext);
					ModelSkeletonUtil.insert(compositeComponent.getRequiringDelegationConnectors(), requiringConnector);
				} else {
					// Local call: Also add an assembly context for the called
					// component within the composite component
					// and connect it with an assembly connector.
					AssemblyContext calledAssembly = addAssemblyContext(compositeComponent, outgoingCrec,
							calledComponent);

					AssemblyConnector connector = RepositoryFactory.eINSTANCE.createAssemblyConnector();
					connector.setName(toName(innerAssemblyContext.getName() + " -> " + calledAssembly.getName()));
					connector.setInterfaceProvidingRole(calledProvidingRole);
					connector.setInterfaceRequiringRole(callerRequiringRole);
					connector.setProvidingAssemblyContext(calledAssembly);
					connector.setRequiringAssemblyContext(innerAssemblyContext);
					ModelSkeletonUtil.insert(compositeComponent.getAssemblyConnectors(), connector);
				}
			}
			callerRequiringRole = remoteRequiringRole;
			callerComponent = compositeComponent;
		}

		// 6. Now create the assembly contexts
		AssemblyContext callerContext = addAssemblyContext(skeleton.getSystem(), incomingCrec, callerComponent);
		if (callerRequiringRole != null) {
			AssemblyContext calledContext = addAssemblyContext(skeleton.getSystem(), outgoingCrec, calledComponent);

			AssemblyConnector connector = RepositoryFactory.eINSTANCE.createAssemblyConnector();
			connector.setName(toName(callerContext.getName() + " -> " + calledContext.getName()));
			connector.setInterfaceProvidingRole(calledProvidingRole);
			connector.setInterfaceRequiringRole(callerRequiringRole);
			connector.setProvidingAssemblyContext(calledContext);
			connector.setRequiringAssemblyContext(callerContext);
			ModelSkeletonUtil.insert(skeleton.getSystem().getAssemblyConnectors(), connector);
		}

		// 7. Expose the provided remote interfaces as system interfaces (if
		// this port is marked as system interface)
		String systemProvidingRoleName = getInterfaceProvidingRoleName(incomingIrec, callerContext.getName());
		// Do not create a new providing/requiring role for the system, only use
		// the existing providing/requiring role.
		// This way a user can control which interfaces are exposed at the
		// system level by creating them manually.
		InterfaceProvidingRole systemProvidingRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
		systemProvidingRole.setName(toName(systemProvidingRoleName));
		systemProvidingRole.setInterface(incomingInterface);
		systemProvidingRole = ModelSkeletonUtil.find(
				((ApplicationScope) getContainerScope()).getApplication().getInterfaceProvidingRoles(),
				systemProvidingRole);
		if (systemProvidingRole != null) {
			systemProvidingRole = ModelSkeletonUtil.adopt(this, systemProvidingRole);
			skeleton.getSystem().getInterfaceProvidingRoles()
					.add(systemProvidingRole);

			ProvidingDelegationConnector systemProvidingConnector = RepositoryFactory.eINSTANCE
					.createProvidingDelegationConnector();
			systemProvidingConnector.setName(toName(callerContext.getName()));
			systemProvidingConnector.setOuterInterfaceProvidingRole(systemProvidingRole);
			systemProvidingConnector.setInnerInterfaceProvidingRole(callerProvidingRole);
			systemProvidingConnector.setAssemblyContext(callerContext);
			ModelSkeletonUtil.insert(skeleton.getSystem().getProvidingDelegationConnectors(), systemProvidingConnector);
		}

		if (callerRequiringRole != null) {
			String systemRequiringRoleName = getInterfaceRequiringRoleName(outgoingIrec, callerContext.getName());
			InterfaceRequiringRole systemRequiringRole = RepositoryFactory.eINSTANCE.createInterfaceRequiringRole();
			systemRequiringRole.setName(toName(systemRequiringRoleName));
			systemRequiringRole.setInterface(outgoingInterface);
			systemRequiringRole = ModelSkeletonUtil.find(
					((ApplicationScope) getContainerScope()).getApplication().getInterfaceRequiringRoles(),
					systemRequiringRole);
			if (systemRequiringRole != null) {
				systemRequiringRole = ModelSkeletonUtil.adopt(this, systemRequiringRole);
				skeleton.getSystem().getInterfaceRequiringRoles()
						.add(ModelSkeletonUtil.adopt(this, systemRequiringRole));

				RequiringDelegationConnector systemRequiringConnector = RepositoryFactory.eINSTANCE
						.createRequiringDelegationConnector();
				systemRequiringConnector.setName(toName(callerContext.getName()));
				systemRequiringConnector.setInnerInterfaceRequiringRole(callerRequiringRole);
				systemRequiringConnector.setOuterInterfaceRequiringRole(systemRequiringRole);
				systemRequiringConnector.setAssemblyContext(callerContext);
				ModelSkeletonUtil.insert(skeleton.getSystem().getRequiringDelegationConnectors(),
						systemRequiringConnector);
			}
		}

		// 8. Add sensor descriptions for application metrics
		List<AssemblyContext> instancePath = new LinkedList<>();
		instancePath.add(callerContext);
		if (innerAssemblyContext != null) {
			instancePath.add(innerAssemblyContext);
		}
		addSensors(incomingIrec, outgoingIrec, instancePath, innerCallerProvidingRole, incomingSignature, externalCall);

		// 9. Remember to deploy this context (only for the caller component)
		undeployedAssemblyContexts.add(callerContext);
	}

	private <D extends Dimension> ComponentSensor<D> createSensor(Metric<D> metric, Unit<D> unit,
			Aggregation aggregation, List<AssemblyContext> instance, InterfaceProvidingRole role, Signature signature,
			EObject position) {
		ComponentInstanceReference ref = ParameterdependenciesFactory.eINSTANCE.createComponentInstanceReference();
		ref.getAssemblies().addAll(instance);
		ComponentSensor<D> sensor = SensorFactory.eINSTANCE.createComponentSensor();
		sensor.setMetric(metric);
		sensor.setUnit(unit);
		sensor.setInterval(Constants.MONITORING_INTERVAL_IN_MILLIS);
		sensor.setAggregation(aggregation);
		sensor.setComponentInstance(ref);
		sensor.setInterfaceProvidingRole(role);
		sensor.setSignature(signature);
		sensor.setPosition(position);
		return sensor;
	}

	private void addSensors(InvocationRecord incomingRecord, InvocationRecord outgoingRecord,
			List<AssemblyContext> componentInstance, InterfaceProvidingRole innerCallerProvidingRole,
			Signature incomingSignature, ExternalCall externalCall) {
		ComponentSensor<RequestCount> inThroughputSensor = createSensor(StandardMetrics.DEPARTURES,
				RequestCount.REQUESTS, Aggregation.CUMULATIVE_SUM, componentInstance,
				innerCallerProvidingRole, incomingSignature, null);
		inThroughputSensor = ModelSkeletonUtil.insert(skeleton.getSensors().getSensors(), inThroughputSensor);
		incomingRecord.setThroughputSensor(inThroughputSensor);

		ComponentSensor<Time> inResponseTimeSensor = createSensor(StandardMetrics.RESPONSE_TIME,
				Time.NANOSECONDS, Aggregation.CUMULATIVE_SUM, componentInstance,
				innerCallerProvidingRole, incomingSignature, null);
		inResponseTimeSensor = ModelSkeletonUtil.insert(skeleton.getSensors().getSensors(), inResponseTimeSensor);
		incomingRecord.setResponseTimeSensor(inResponseTimeSensor);

		ComponentSensor<Time> inResidenceTimeSensor = createSensor(StandardMetrics.RESIDENCE_TIME, Time.NANOSECONDS,
				Aggregation.CUMULATIVE_SUM, componentInstance, innerCallerProvidingRole, incomingSignature, null);
		inResidenceTimeSensor = ModelSkeletonUtil.insert(skeleton.getSensors().getSensors(), inResidenceTimeSensor);
		incomingRecord.setResidenceTimeSensor(inResidenceTimeSensor);

		if (externalCall != null) {
			ComponentSensor<RequestCount> outThroughputSensor = createSensor(StandardMetrics.DEPARTURES,
					RequestCount.REQUESTS, Aggregation.CUMULATIVE_SUM, componentInstance,
					innerCallerProvidingRole, incomingSignature, externalCall);
			outThroughputSensor = ModelSkeletonUtil.insert(skeleton.getSensors().getSensors(), outThroughputSensor);
			outgoingRecord.setThroughputSensor(outThroughputSensor);

			ComponentSensor<Time> outResponseTimeSensor = createSensor(StandardMetrics.RESPONSE_TIME, Time.NANOSECONDS,
					Aggregation.CUMULATIVE_SUM, componentInstance, innerCallerProvidingRole, incomingSignature,
					externalCall);
			outResponseTimeSensor = ModelSkeletonUtil.insert(skeleton.getSensors().getSensors(), outResponseTimeSensor);
			outgoingRecord.setResponseTimeSensor(outResponseTimeSensor);
		}
	}

	private void addLocalCallToModule(CompositeComponent module, InterfaceProvidingRole externalRole,
			ComponentRecord callee, InvocationRecord outgoingIrec) {
		// Add an implementation component representing the component that is
		// called.
		BasicComponent calledBasicComponent = RepositoryFactory.eINSTANCE.createBasicComponent();
		calledBasicComponent.setName(toName(callee.getComponentClass()));
		calledBasicComponent = ModelSkeletonUtil.insert(skeleton.getRepository().getComponents(), calledBasicComponent);

		// Add an interface providing role
		String providingPortName = getInterfaceProvidingRoleName(outgoingIrec, "");
		InterfaceProvidingRole callerProvidingRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
		callerProvidingRole.setName(toName(providingPortName));
		callerProvidingRole.setInterface(externalRole.getInterface());
		callerProvidingRole = ModelSkeletonUtil.insert(calledBasicComponent.getInterfaceProvidingRoles(),
				callerProvidingRole);

		// Create the assembly connections inside the module composite
		// component.
		AssemblyContext calledAssembly = addAssemblyContext(module, callee, calledBasicComponent);

		ProvidingDelegationConnector providingConnector = RepositoryFactory.eINSTANCE
				.createProvidingDelegationConnector();
		providingConnector.setName(toName(calledAssembly.getName()));
		providingConnector.setOuterInterfaceProvidingRole(externalRole);
		providingConnector.setInnerInterfaceProvidingRole(callerProvidingRole);
		providingConnector.setAssemblyContext(calledAssembly);
		ModelSkeletonUtil.insert(module.getProvidingDelegationConnectors(), providingConnector);
	}

	// This creates a basic component that represents the component service for
	// a database access
	private void addDataSource(CompositeComponent dataSourceComponent, InterfaceProvidingRole role,
			Signature signature) {
		BasicComponent dataBaseComponent = RepositoryFactory.eINSTANCE.createBasicComponent();
		dataBaseComponent.setName("database");
		dataBaseComponent = ModelSkeletonUtil.insert(skeleton.getRepository().getComponents(), dataBaseComponent);

		InterfaceProvidingRole innerProvidingRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
		innerProvidingRole.setInterface(role.getInterface());
		innerProvidingRole.setName(toName("jdbc://javax.sql.DataSource"));
		innerProvidingRole = ModelSkeletonUtil.insert(dataBaseComponent.getInterfaceProvidingRoles(),
				innerProvidingRole);

		AssemblyContext databaseContext = RepositoryFactory.eINSTANCE.createAssemblyContext();
		databaseContext.setName("database");
		databaseContext.setEncapsulatedComponent(dataBaseComponent);
		databaseContext = ModelSkeletonUtil.insert(dataSourceComponent.getAssemblyContexts(), databaseContext);

		ProvidingDelegationConnector delegation = RepositoryFactory.eINSTANCE.createProvidingDelegationConnector();
		delegation.setName("database");
		delegation.setAssemblyContext(databaseContext);
		delegation.setInnerInterfaceProvidingRole(innerProvidingRole);
		delegation.setOuterInterfaceProvidingRole(role);
		ModelSkeletonUtil.insert(dataSourceComponent.getProvidingDelegationConnectors(), delegation);

		createComponentInternalBehavior(dataBaseComponent, innerProvidingRole, signature);
	}

	private FineGrainedBehavior createComponentInternalBehavior(BasicComponent implementationComponent,
			InterfaceProvidingRole implementationProvidingRole, Signature signature) {
		FineGrainedBehavior behavior = ServicebehaviorFactory.eINSTANCE.createFineGrainedBehavior();
		behavior.setDescribedSignature(signature);
		behavior.setInterfaceProvidingRole(implementationProvidingRole);
		behavior = ModelSkeletonUtil.insert(implementationComponent.getFineGrainedBehavior(), behavior);
		if (behavior.getBehavior() == null) {
			behavior.setBehavior(ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior());
		}
		if (!isJMSComponent(implementationComponent.getName())) {
			if (behavior.getBehavior().getActions().isEmpty()) {
				InternalAction internalAction = ServicebehaviorFactory.eINSTANCE.createInternalAction();
				ResourceDemand cpuDemand = ServicebehaviorFactory.eINSTANCE.createResourceDemand();
				cpuDemand.setName("CPU");
				cpuDemand.setCharacterization(ModelVariableCharacterizationType.EMPIRICAL);
				cpuDemand.setResourceType(
						ModelSkeletonUtil.adopt(this, getContainerScope().getSystemScope().getCpuResourceType()));
				internalAction.getResourceDemand().add(cpuDemand);
				ModelSkeletonUtil.insert(behavior.getBehavior().getActions(), internalAction);
			}
		}
		return behavior;
	}

	private AssemblyContext addAssemblyContext(ComposedStructure parent, ComponentRecord componentRecord,
			RepositoryComponent component) {
		AssemblyContext context = RepositoryFactory.eINSTANCE.createAssemblyContext();
		StringBuilder contextName = new StringBuilder();
		if (!(parent instanceof CompositeComponent)) {
			// In order to uniquely identify the system level assembly contexts,
			// we also include the server node name (e.g., in case of
			// replicated tiers)
			contextName.append(componentRecord.getServerNode());
			contextName.append("/");
		}
		if (component instanceof CompositeComponent) {
			// This component is the container for more basic components
			// use application and module in name
			contextName.append(componentRecord.getApplication());
			if (componentRecord.getModule() != null && !componentRecord.getModule().isEmpty()) {
				contextName.append("/");
				contextName.append(componentRecord.getModule());
			}
		} else {
			contextName.append(componentRecord.getComponentName());
		}
		context.setName(toName(contextName.toString()));
		context.setEncapsulatedComponent(component);
		return ModelSkeletonUtil.insert(parent.getAssemblyContexts(), context);
	}

	private ExternalCall addExternalCall(String componentName, BasicComponent implementationComponent,
			FineGrainedBehavior behavior, InterfaceRequiringRole requiringRole, Signature outgoingSignature,
			InvocationRecord outgoingInvocation) {
		ExternalCallAction callAction = ServicebehaviorFactory.eINSTANCE.createExternalCallAction();
		ExternalCall call = ServicebehaviorFactory.eINSTANCE.createExternalCall();
		call.setName(toName(requiringRole.getName() + "#" + outgoingSignature.getName()));
		call.setSignature(outgoingSignature);
		call.setInterfaceRequiringRole(requiringRole);
		callAction.setExternalCall(call);

		if (isJMSComponent(toName(componentName))) {
			if (behavior.getBehavior().getActions().isEmpty()) {
				ForkAction fork = ServicebehaviorFactory.eINSTANCE.createForkAction();
				fork.setWithSynchronizationBarrier(false);
				ComponentInternalBehavior forkedBehavior = ServicebehaviorFactory.eINSTANCE
						.createComponentInternalBehavior();
				forkedBehavior.getActions().add(callAction);
				fork.getForkedBehaviors().add(forkedBehavior);
				behavior.getBehavior().getActions().add(fork);
				return call;
			} else {
				return null;
			}
		} else {
			LoopAction loop = ServicebehaviorFactory.eINSTANCE.createLoopAction();
			LoopIterationCount iterationCount = ServicebehaviorFactory.eINSTANCE.createLoopIterationCount();
			iterationCount.setName(toName(call.getName()));
			iterationCount.setCharacterization(ModelVariableCharacterizationType.EMPIRICAL);
			loop.setLoopIterationCount(iterationCount);
			loop.setLoopBodyBehavior(ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior());

			// We may add delay actions to represent the network delays for
			// remote external calls.
			if (Constants.ADD_NETWORK_DELAYS) {
				if (isRemoteCall(outgoingInvocation)) {
					InternalAction delayAction = ServicebehaviorFactory.eINSTANCE.createInternalAction();
					ResourceDemand demand = ServicebehaviorFactory.eINSTANCE.createResourceDemand();
					demand.setCharacterization(ModelVariableCharacterizationType.EMPIRICAL);
					demand.setName("Network");
					demand.setResourceType(
							ModelSkeletonUtil.adopt(this, getContainerScope().getSystemScope().getDelayResourceType()));
					delayAction.getResourceDemand().add(demand);
					loop.getLoopBodyBehavior().getActions().add(delayAction);
				}
			}
			loop.getLoopBodyBehavior().getActions().add(callAction);

			// We assume that the external calls are inserted in
			// alphabetical order.
			EList<AbstractAction> actions = behavior.getBehavior().getActions();
			for (int i = 0; i < actions.size(); i++) {
				if (actions.get(i) instanceof LoopAction) {
					LoopAction curLoop = (LoopAction) actions.get(i);
					int compareResult = curLoop.getLoopIterationCount().getName().compareTo(call.getName());
					if (compareResult == 0) {
						return null;
					} else if (compareResult > 0) {
						actions.add(i, loop);
						return call;
					}
				}
			}
			// The previous loop did not return, i.e., the new action is added
			// at the back.
			actions.add(loop);
			return call;
		}
	}

	private boolean isRemoteCall(InvocationRecord outgoingInvocation) {
		return !outgoingInvocation.getOperation().getProtocol().equals("java");
	}

	private CompositeComponent createCompositeComponent(ComponentRecord crec,
			RepositoryComponent implementationComponent) {
		String compositeName = getCompositeComponentName(crec);
		if (compositeName.length() > 0) {
			CompositeComponent compositeComponent = RepositoryFactory.eINSTANCE.createCompositeComponent();
			compositeComponent.setName(toName(compositeName.toString()));
			return (CompositeComponent) ModelSkeletonUtil.insert(skeleton.getRepository().getComponents(),
					compositeComponent);
		}
		return null;
	}

	private RepositoryComponent createCalledComponent(ComponentRecord callee,
			boolean remoteCall) {
		// check whether this call goes to a component in the same composite or
		// not
		if (remoteCall) {
			String calleeCompositeName = getCompositeComponentName(callee);
			if (calleeCompositeName.length() > 0) {
				CompositeComponent compositeComponent = RepositoryFactory.eINSTANCE.createCompositeComponent();
				compositeComponent.setName(toName(calleeCompositeName));
				return (CompositeComponent) ModelSkeletonUtil.insert(skeleton.getRepository().getComponents(),
						compositeComponent);
			}
		}
		if (callee.getComponentClass() != null) {
			BasicComponent basicComponent = RepositoryFactory.eINSTANCE.createBasicComponent();
			basicComponent.setName(toName(callee.getComponentClass()));
			return (BasicComponent) ModelSkeletonUtil.insert(skeleton.getRepository().getComponents(), basicComponent);
		}
		throw new IllegalStateException("Could not create called component for " + callee.getComponentName());
	}

	private boolean isDifferentModule(ComponentRecord caller, ComponentRecord callee) {
		boolean sameModule = caller.getApplication().equals(callee.getApplication())
				&& caller.getModule().equals(callee.getModule());
		return !sameModule;
	}

	private boolean isJMSComponent(String componentName) {
		return toName("javax.jms.Queue").equals(componentName) || toName("javax.jms.Topic").equals(componentName);
	}

	private String getCompositeComponentName(ComponentRecord crec) {
		StringBuilder compositeName = new StringBuilder();
		if (crec.getApplication() != null && !crec.getApplication().isEmpty()) {
			compositeName.append(crec.getApplication());
		}
		if (crec.getModule() != null && !crec.getModule().isEmpty()) {
			if (compositeName.length() > 0) {
				compositeName.append("/");
			}
			compositeName.append(crec.getModule());
		}
		return compositeName.toString();
	}

	private int getRequiringPortIndex(ComponentRecord incomingCrec, OperationRecord outgoingIrec) {
		CallPaths paths = callpaths.get(incomingCrec.getComponentName());
		if (paths == null) {
			paths = new CallPaths();
			callpaths.put(incomingCrec.getComponentName(), paths);
		}
		paths.addPath(outgoingIrec.getInterfaceName(), outgoingIrec.getComponent().getComponentName());
		int idx = paths.getIndex(outgoingIrec.getInterfaceName(), outgoingIrec.getComponent().getComponentName());
		if (idx < 0) {
			throw new IllegalStateException();
		}
		return idx;
	}

	public InjectedValue<MonitoringService> getMonitoringService() {
		return monitoring;
	}

	private String getSignatureName(InvocationRecord record) {
		String name = record.getOperation().getInterfaceName() + "#"
				+ record.getOperation().getOperationName();
		if (record.getOperation().areParametersMonitored()) {
			name += "_" + record.getValueBucket();
		}
		return name;
	}

	private String getInterfaceProvidingRoleName(InvocationRecord record, String assembly) {
		StringBuilder res = new StringBuilder();
		res.append(record.getOperation().getProtocol());
		res.append("://");
		if (!assembly.isEmpty()) {
			res.append(assembly);
			res.append("/");
		}
		res.append(record.getOperation().getInterfaceName());
		return res.toString();
	}

	private String getInterfaceRequiringRoleName(InvocationRecord record, String assembly) {
		StringBuilder res = new StringBuilder();
		res.append(record.getOperation().getProtocol());
		res.append("://");
		if (!assembly.isEmpty()) {
			res.append(assembly);
			res.append("/");
		}
		res.append(record.getOperation().getInterfaceName());
		return res.toString();
	}

	private String toName(String name) {
		// The DML text editors do not like colons in names.
		return name.replace(".", "_");
	}
}
